###############################################################################
##                      Reproducibility code for:                             #
# Low vertical transfer rates of carbon inferred from radiocarbon analysis    #
#                          in an Amazon podzol                                #
###############################################################################
# Created by Carlos A. Sierra
################################################  January, 2013
###############################################################################

# 1. Load required libraries, read dataset, and create data-objects
library(SoilR) #Load SoilR library

zar=read.csv("/Users/_csierra/Documents/Submissions/2013/Zafire14C/CodeData/ZAR14C.csv") #Change to location where you stored accompanying file
data(C14Atm_NH) # Add post-bomb radiocarbon dataset

# Transform the pM (14aN) values of the bulk soil to Delta14C.
sy=2011 #Sampling year
soil.D14C<-(((zar[,14]/100)*exp(-(sy-1950)/8267))-1)*1000

zar=cbind(zar,soil.D14C)

# Topsoil data
Plot=zar[c(4:11,17:24),]
#Profile data
Profile=zar[c(1:3,12:16),]
Prof=data.frame(label=c("Z04 C1","Z04 C2","Z04 C3","Z01 C2","Z01 C1","Z01 C3"),
                Depth=c(Profile[1:6,5]),
                C=c(Profile[1:5,12],mean(Profile[6:8,12])),
                Rh=c(Profile[1:5,9],mean(Profile[6:8,9])),
                Rh_D14C=c(Profile[1:5,7],mean(Profile[6:8,7])),
                Rh_D14Csd=c(Profile[1:5,8],mean(Profile[6:8,8])),
                Bk_D14C=c(Profile[1:5,16],mean(Profile[6:8,16]))
)
Prof=cbind(Prof,Bk_D14Csd=abs(Prof$Rh_D14Csd*Prof$Bk_D14C/Prof$Rh_D14C))

###############################################################################
# 2. Topsoil

#Statistical comparisons
Clm=lm(C~Plot*Horizon,data=Plot)
anova(Clm)
Nlm=lm(N~Plot*Horizon,data=Plot)
anova(Nlm)
S14lm=lm(soil.D14C~Plot*Horizon,data=Plot)
anova(S14lm)
R14lm=lm(Rh_DEL14C~Plot*Horizon,data=Plot)
anova(R14lm)

t.test(x=Plot$Rh_DEL14C,y=Plot$soil.D14C)

C14lm=lm((soil.D14C-Rh_DEL14C)~Plot,data=Plot)
anova(C14lm)

# Figure 1.
par(mfrow=c(2,2))
boxplot(C*10~Plot*Horizon,data=zar,ylim=c(0,500),xlim=c(6.6,10.5),ylab=expression(paste("C concentration (mg g"^-1,")")),main="a")
boxplot(N*10~Plot*Horizon,data=zar,ylim=c(0,50),xlim=c(6.6,10.5),ylab=expression(paste("N concentration (mg g"^-1,")")),main="b")
boxplot(soil.D14C~Plot*Horizon,data=Plot,ylim=c(0,200),xlim=c(6.6,10.5),ylab=expression(paste(Delta^14,"C (\u2030)")),main="c")
abline(h=tail(C14Atm_NH, n=1L)[2],lty=2,col=4)
boxplot(Rh_DEL14C~Plot*Horizon,data=Plot,ylim=c(0,200),xlim=c(6.6,10.5),ylab=expression(paste(Delta^14,"C (\u2030)")),main="d")
abline(h=tail(C14Atm_NH, n=1L)[2],lty=2,col=4)
par(mfrow=c(1,1))

###############################################################################
# 3. Subsoil

# Figure 2. C content and Rh profile
par(mfrow=c(2,1),mar=c(4,5,1,3))
plot(Prof$C[1:3]*10,Prof$Depth[1:3],type="p",ylim=c(-70,0),xlab=expression(paste("C concentration (mg g"^-1,")")),ylab="Depth (m)")
points(Prof$C[4:6]*10,Prof$Depth[4:6],type="p",col=2,pch=19)
segments(0,Prof$Depth[1:3],Prof$C[1:3]*10,Prof$Depth[1:3],lwd=2)
segments(0,Prof$Depth[4:6],Prof$C[4:6]*10,Prof$Depth[4:6],col=2,lwd=2)

plot(Prof$Rh[1:3],Prof$Depth[1:3],type="p",ylim=c(-70,0),xlab=expression(paste("Rh (mg C gdw"^-1, "day"^-1,")")),ylab="Depth (m)")
points(Prof$Rh[4:6],Prof$Depth[4:6],type="p",col=2,pch=19)
segments(0,Prof$Depth[1:3],Prof$Rh[1:3],Prof$Depth[1:3],lwd=2)
segments(0,Prof$Depth[4:6],Prof$Rh[4:6],Prof$Depth[4:6],col=2,lwd=2)
legend("bottomright",c("ZAR-04", "ZAR-01"),lty=1,pch=c(1,19),col=c(1,2),bty="n")
par(mfrow=c(1,1))

#Figure 3. Radiocarbon profiles
plot(Prof$Rh_D14C[1:3],Prof$Depth[1:3],type="b",xlim=c(-250,120),ylim=c(-100,0),ylab="Depth (m)",xlab=expression(paste(Delta^14,"C (\u2030)")))
arrows(Prof$Rh_D14C[1:3]+Prof$Rh_D14Csd[1:3],Prof$Depth[1:3],Prof$Rh_D14C[1:3]-Prof$Rh_D14Csd[1:3],Prof$Depth[1:3], angle=90, code=3,length=0.1)
points(Prof$Rh_D14C[4:6],Prof$Depth[4:6],type="b",col=2,pch=19)
arrows(Prof$Rh_D14C[4:6]+Prof$Rh_D14Csd[4:6],Prof$Depth[4:6],Prof$Rh_D14C[4:6]-Prof$Rh_D14Csd[4:6],Prof$Depth[4:6], angle=90, code=3,length=0.1,col=2)
abline(v=0,lty=2)
points(Prof$Bk_D14C[4:6],Prof$Depth[4:6],type="b",col=2,lwd=2.5,pch=19)
arrows(Prof$Bk_D14C[4:6]+Prof$Bk_D14Csd[4:6],Prof$Depth[4:6],Prof$Bk_D14C[4:6]-Prof$Bk_D14Csd[4:6],Prof$Depth[4:6], angle=90, code=3,length=0.1,col=2)
points(Prof$Bk_D14C[1:3],Prof$Depth[1:3],type="b",lwd=2.5)
arrows(Prof$Bk_D14C[1:3]+Prof$Bk_D14Csd[1:3],Prof$Depth[1:3],Prof$Bk_D14C[1:3]-Prof$Bk_D14Csd[1:3],Prof$Depth[1:3], angle=90, code=3,length=0.1)
legend("bottomleft",c("Soil C ZAR-01",expression(paste("Respired CO"[2], " ZAR-01")),"Soil C ZAR-04",expression(paste("Respired CO"[2], " ZAR-04"))),lty=c(1,1,1,1),lwd=c(2.5,1),pch=c(19,19,1,1),col=c(2,2,rep(1,2)),bty="n")

###############################################################################
# 4. Modeling

# Intcal dataset (pre-bomb dataset), it downloads directly from Radiocarbon website
IntCal09=read.table("http://www.radiocarbon.org/IntCal09%20files/intcal09.14c",skip=10,sep=",",
                    col.names=c("CAL BP", "C14 age","Error","Delta 14C","Sigma"))

#Create new dataset with pre- and post-bomb data
C14d=data.frame(AD=c(1950-IntCal09[1:(dim(IntCal09)[1]-1),1],C14Atm_NH[52:111,1]),D14C=c(IntCal09[1:(dim(IntCal09)[1]-1),4],C14Atm_NH[52:111,2]))

#Create model as an object. See manuscript for model description
ZafireModel=function(t, ks,  C0, In, a31,  a42, g, xi=1, FcAtm, lambda=-0.0001209681, lag=0, solver=deSolve.lsoda.wrapper, pass=FALSE){
  t_start=min(t)
  t_stop=max(t)
  if(length(ks)!=4) stop("ks must be of length = 4")
  if(length(C0)!=4) stop("the vector with initial conditions must be of length = 4")
  
  if(length(In)==1) inputFluxes=new("TimeMap",
                                    t_start,
                                    t_stop,
                                    function(t){matrix(nrow=4,ncol=1,c(In*g,In*(1-g),0,0))}
  )
  
  if(length(In)==4) inputFluxes=new("TimeMap",
                                    t_start,
                                    t_stop,
                                    function(t){matrix(nrow=3,ncol=1,c(In[1],In[2],In[3],In[4]))}
  )
  
  if(class(In)=="data.frame"){
    x=In[,1]  
    y=In[,2]  
    inputFlux=function(t0){as.numeric(spline(x,y,xout=t0)[2])}
    inputFluxes=new("TimeMap",
                    t_start,
                    t_stop,
                    function(t){matrix(nrow=4,ncol=1,c(inputFlux(t),0,0,0))}
    )   
  }
  
  if(class(In)=="TimeMap") inputFluxes=In
  
  if(length(xi)==1) fX=function(t){xi}
  if(class(xi)=="data.frame"){
    X=xi[,1]
    Y=xi[,2]
    fX=function(t){as.numeric(spline(X,Y,xout=t)[2])}
  }
  
  A=-abs(diag(ks))
  A[3,1]=a31
  A[4,2]=a42
  
  At=new(Class="DecompositionOperator",
         t_start,
         t_stop,
         function(t){
           fX(t)*A
         }
  ) 
  
  Fc=FcAtm.from.Dataframe(FcAtm,lag,format="Delta14C")
  
  mod=GeneralModel_14(t,At,ivList=C0,inputFluxes=inputFluxes,Fc,di=lambda)
}	

#### Run model

#Parameters for the terra-firme forest (ZAR-04)
years=seq(-1000,2010,by=0.5)
IZ4=4.72+1.53 # Litter inputs for ZAR-04 
ksZ4=c(k1=2, k2=0.1, k3=1, k4=1/5000) #Decomposition rates

#Run the model and calcuate radiocarbon in C and Rh 
Z4=ZafireModel(t=years,ks=ksZ4, C0=c(20,20,20,47), In=IZ4, g=0.1, a31=0.9*ksZ4[1], a42=0.003*ksZ4[2], FcAtm=C14d)
RtZ4=getReleaseFlux(Z4)
C14Z4=getC14(Z4)
C14pZ4=getF14(Z4)
CtZ4=Z4["C"]
R14mZ4=getReleaseFlux14(Z4)
R14topZ4=Delta14C_from_AbsoluteFractionModern(rowSums(R14mZ4[,1:2])/rowSums(RtZ4[,1:2]))
R14subZ4=Delta14C_from_AbsoluteFractionModern(rowSums(R14mZ4[,3:4])/rowSums(RtZ4[,3:4]))
C14subZ4=Delta14C_from_AbsoluteFractionModern(rowSums(C14Z4[,3:4])/rowSums(CtZ4[,3:4]))

#The vertical transfers are calcuated as
VtransferZ4=RtZ4[,1:2]%*%c(0.9*ksZ4[1], 0.003*ksZ4[2])


#Parameters for the varillal (ZAR-01)
IZ1=2.48+2.98 #Litter inputs for ZAR-01
ksZ1=c(k1=2, k2=0.11, k3=1, k4=1/5000)

Z1=ZafireModel(t=years,ks=ksZ1, C0=c(20,20,20,47), In=IZ1, g=0.1, a31=0.017*ksZ1[1], a42=0.0025*ksZ1[2], FcAtm=C14d)
RtZ1=getReleaseFlux(Z1)
C14Z1=getC14(Z1)
C14pZ1=getF14(Z1)
CtZ1=Z1["C"]
R14mZ1=getReleaseFlux14(Z1)
R14topZ1=Delta14C_from_AbsoluteFractionModern(rowSums(R14mZ1[,1:2])/rowSums(RtZ1[,1:2]))
R14subZ1=Delta14C_from_AbsoluteFractionModern(rowSums(R14mZ1[,3:4])/rowSums(RtZ1[,3:4]))
C14subZ1=Delta14C_from_AbsoluteFractionModern(rowSums(C14Z1[,3:4])/rowSums(CtZ1[,3:4]))

#The vertical transfers are calcuated as
VtransferZ1=RtZ1[,1:2]%*%c(0.017*ksZ1[1], 0.0025*ksZ1[2])

######## Figure 4
par(mfrow=c(2,1), mar=c(5, 5, 1, 2))
plot(C14d,type="l",xlab="Year",ylab=expression(paste(Delta^14,"C (\u2030)")),xlim=c(1950,2010),ylim=c(-200,1000)) 
lines(years, R14topZ4,col=4)
lines(years,R14subZ4,col=2)
lines(years,C14subZ4,col=3)
points(2010,Prof[1,5],col=4,pch=20)
arrows(2010,Prof[1,5]+Prof[1,6],2010,Prof[1,5]-Prof[1,6],angle=90,code=3,length=0.1,col=4)
points(2010,Prof[3,5],col=2,pch=20)
arrows(2010,Prof[3,5]+Prof[3,6],2010,Prof[3,5]-Prof[3,6],angle=90,code=3,length=0.1,col=2)
points(2010,Prof[3,7],col=3,pch=20)
arrows(2010,Prof[3,7]+Prof[3,8],2010,Prof[3,7]-Prof[3,8],angle=90,code=3,length=0.1,col=3)
legend("topright",c(expression(paste(Delta^14,"C atmosphere")), expression(paste(Delta^14,"C-",C0[2], " from topsoil")), 
                    expression(paste(Delta^14,"C-",C0[2], " from subsoil")), expression(paste(Delta^14,"C in SOM from subsoil"))),
       lty=1,col=c(1,4,2,3),bty="n")
legend("topleft",legend="a",bty="n")

plot(C14d,type="l",xlab="Year",ylab=expression(paste(Delta^14,"C (\u2030)")),xlim=c(1950,2010),ylim=c(-200,1000)) 
lines(years, R14topZ1,col=4)
lines(years,R14subZ1,col=2)
lines(years,C14subZ1,col=3)
points(2010,Prof[5,5],col=4,pch=20)
arrows(2010,Prof[5,5]+Prof[5,6],2010,Prof[5,5]-Prof[5,6],angle=90,code=3,length=0.1,col=4)
points(2010,Prof[6,5],col=2,pch=20)
arrows(2010,Prof[6,5]+Prof[6,6],2010,Prof[6,5]-Prof[6,6],angle=90,code=3,length=0.1,col=2)
points(2010,Prof[6,7],col=3,pch=20)
arrows(2010,Prof[6,7]+Prof[6,8],2010,Prof[6,7]-Prof[6,8],angle=90,code=3,length=0.1,col=3)
legend("topright",c(expression(paste(Delta^14,"C atmosphere")), expression(paste(Delta^14,"C-",C0[2], " from topsoil")), 
                    expression(paste(Delta^14,"C-",C0[2], " from subsoil")), expression(paste(Delta^14,"C in SOM from subsoil"))),
       lty=1,col=c(1,4,2,3),bty="n")
legend("topleft",legend="b",bty="n")
par(mfrow=c(1,1))
################################################################################
#Runs on R version 2.15.1 (2012-06-22) and SoilR version 1.0-4
#End.

